@@ -29,7 +29,6 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <math.h>
-#include <gconf/gconf-client.h>
 
 #include <libdbusmenu-gtk/menuitem.h>
 #include <libdbusmenu-glib/server.h>
@@ -40,11 +39,9 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <geoclue/geoclue-master-client.h>
 
 #include <time.h>
-#include <libecal/e-cal.h>
+#include <libecal/libecal.h>
 #include <libical/ical.h>
-#include <libecal/e-cal-time-util.h>
-#include <libedataserver/e-source.h>
-#include <libedataserverui/e-passwords.h>
+#include <libedataserver/libedataserver.h>
 // Other users of ecal seem to also include these, not sure why they should be included by the above
 #include <libical/icaltime.h>
 #include <cairo/cairo.h>
@@ -54,6 +51,13 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "settings-shared.h"
 #include "utils.h"
 
+/* how often to check for clock skew */
+#define SKEW_CHECK_INTERVAL_SEC 10
+
+#define MAX_APPOINTMENT_MENUITEMS 5
+
+#define SKEW_DIFF_THRESHOLD_SEC (SKEW_CHECK_INTERVAL_SEC + 5)
+
 #ifdef HAVE_CCPANEL
  #define SETTINGS_APP_INVOCATION "gnome-control-center indicator-datetime"
 #else
@@ -63,9 +67,8 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 static void geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar * path, GError * error, gpointer user_data);
 static gboolean update_appointment_menu_items (gpointer user_data);
 static void update_location_menu_items (void);
-static void setup_timer (void);
+static void day_timer_reset (void);
 static void geo_client_invalid (GeoclueMasterClient * client, gpointer user_data);
-static void geo_address_change (GeoclueMasterClient * client, gchar * a, gchar * b, gchar * c, gchar * d, gpointer user_data);
 static gboolean get_greeter_mode (void);
 
 static void quick_set_tz (DbusmenuMenuitem * menuitem, guint timestamp, gpointer user_data);
@@ -83,13 +86,14 @@ static DbusmenuMenuitem * settings = NULL;
 static DbusmenuMenuitem * events_separator = NULL;
 static DbusmenuMenuitem * locations_separator = NULL;
 static DbusmenuMenuitem * add_appointment = NULL;
-static GList            * appointments = NULL;
+static DbusmenuMenuitem * appointments[MAX_APPOINTMENT_MENUITEMS];
 static GSList           * location_menu_items = NULL;
 static GList            * comp_instances = NULL;
 static gboolean           updating_appointments = FALSE;
 static time_t             start_time_appointments = (time_t) 0;
 static GSettings        * conf = NULL;
-static GConfClient      * gconf = NULL;
+static ESourceRegistry  * source_registry = NULL;
+static GList            * appointment_sources = NULL;
 
 
 /* Geoclue trackers */
@@ -115,6 +119,7 @@ struct TimeLocation
 	gint32 offset;
 	gchar * zone;
 	gchar * name;
+	gboolean visible;
 };
 static void
 time_location_free (struct TimeLocation * loc)
@@ -124,7 +129,7 @@ time_location_free (struct TimeLocation * loc)
 	g_free (loc);
 }
 static struct TimeLocation*
-time_location_new (const char * zone, const char * name, time_t now)
+time_location_new (const char * zone, const char * name, gboolean visible, time_t now)
 {
 	struct TimeLocation * loc = g_new (struct TimeLocation, 1);
 	GTimeZone * tz = g_time_zone_new (zone);
@@ -132,6 +137,7 @@ time_location_new (const char * zone, const char * name, time_t now)
 	loc->offset = g_time_zone_get_offset (tz, interval);
 	loc->zone = g_strdup (zone);
 	loc->name = g_strdup (name);
+	loc->visible = visible;
 	g_time_zone_unref (tz);
 	g_debug ("%s zone '%s' name '%s' offset is %d", G_STRLOC, zone, name, (int)loc->offset);
 	return loc;
@@ -139,22 +145,22 @@ time_location_new (const char * zone, const char * name, time_t now)
 static int
 time_location_compare (const struct TimeLocation * a, const struct TimeLocation * b)
 {
-	int ret;
-	if (a->offset != b->offset) /* primary key s.t. we can sort by timezone order */
-		ret = a->offset - b->offset;
-	else
+	int ret = a->offset - b->offset; /* primary key */
+	if (!ret)
 		ret = g_strcmp0 (a->name, b->name); /* secondary key */
+	if (!ret)
+		ret = a->visible - b->visible; /* tertiary key */
 	g_debug ("%s comparing '%s' (%d) to '%s' (%d), returning %d", G_STRLOC, a->name, (int)a->offset, b->name, (int)b->offset, ret);
 	return ret;
 }
 static GSList*
-locations_add (GSList * locations, const char * zone, const char * name, time_t now)
+locations_add (GSList * locations, const char * zone, const char * name, gboolean visible, time_t now)
 {
-	struct TimeLocation * loc = time_location_new (zone, name, now);
+	struct TimeLocation * loc = time_location_new (zone, name, visible, now);
 
 	if (g_slist_find_custom (locations, loc, (GCompareFunc)time_location_compare) == NULL) {
 		g_debug ("%s Adding zone '%s', name '%s'", G_STRLOC, zone, name);
-		locations = g_slist_prepend (locations, loc);
+		locations = g_slist_append (locations, loc);
 	} else {
 		g_debug("%s Skipping duplicate zone '%s' name '%s'", G_STRLOC, zone, name);
 		time_location_free (loc);
@@ -188,15 +194,17 @@ update_location_menu_items (void)
 
 	/* maybe add geo_timezone */
 	if (geo_timezone != NULL) {
+		const gboolean visible = g_settings_get_boolean (conf, SETTINGS_SHOW_DETECTED_S);
 		gchar * name = get_current_zone_name (geo_timezone);
-		locations = locations_add (locations, geo_timezone, name, now);
+		locations = locations_add (locations, geo_timezone, name, visible, now);
 		g_free (name);
 	}
 
 	/* maybe add current_timezone */
 	if (current_timezone != NULL) {
+		const gboolean visible = g_settings_get_boolean (conf, SETTINGS_SHOW_DETECTED_S);
 		gchar * name = get_current_zone_name (current_timezone);
-		locations = locations_add (locations, current_timezone, name, now);
+		locations = locations_add (locations, current_timezone, name, visible, now);
 		g_free (name);
 	}
 
@@ -205,12 +213,13 @@ update_location_menu_items (void)
 	if (user_locations != NULL) { 
 		gint i;
 		const guint location_count = g_strv_length (user_locations);
+		const gboolean visible = g_settings_get_boolean (conf, SETTINGS_SHOW_LOCATIONS_S);
 		g_debug ("%s Found %u user-specified locations", G_STRLOC, location_count);
 		for (i=0; i<location_count; i++) {
 			gchar * zone;
 			gchar * name;
 			split_settings_location (user_locations[i], &zone, &name);
-			locations = locations_add (locations, zone, name, now);
+			locations = locations_add (locations, zone, name, visible, now);
 			g_free (name);
 			g_free (zone);
 		}
@@ -218,13 +227,10 @@ update_location_menu_items (void)
 		user_locations = NULL;
 	}
 
-	/* sort the list by timezone offset */
-	locations = g_slist_sort (locations, (GCompareFunc)time_location_compare);
-
 	/* finally create menuitems for each location */
 	gint offset = dbusmenu_menuitem_get_position (locations_separator, root)+1;
-	const gboolean show_locations = g_settings_get_boolean (conf, SETTINGS_SHOW_LOCATIONS_S);
 	GSList * l;
+	gboolean have_visible_location = FALSE;
 	for (l=locations; l!=NULL; l=l->next) {
 		struct TimeLocation * loc = l->data;
 		g_debug("%s Adding location: zone '%s', name '%s'", G_STRLOC, loc->zone, loc->name);
@@ -234,17 +240,19 @@ update_location_menu_items (void)
 		dbusmenu_menuitem_property_set      (item, TIMEZONE_MENUITEM_PROP_ZONE, loc->zone);
 		dbusmenu_menuitem_property_set_bool (item, TIMEZONE_MENUITEM_PROP_RADIO, FALSE);
 		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
-		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, show_locations);
+		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, loc->visible);
 		dbusmenu_menuitem_child_add_position (root, item, offset++);
 		g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
 		location_menu_items = g_slist_append (location_menu_items, item);
+		if (loc->visible)
+			have_visible_location = TRUE;
 		time_location_free (loc);
 	}
 	g_slist_free (locations);
 	locations = NULL;
 
 	/* if there's at least one item being shown, show the separator too */
-	dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, show_locations && (location_menu_items!=NULL));
+	dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, have_visible_location);
 }
 
 /* Update the current timezone */
@@ -354,7 +362,7 @@ update_datetime (gpointer user_data)
 	g_date_time_unref (datetime);
 	g_free(utf8);
 
-	return FALSE;
+	return G_SOURCE_REMOVE;
 }
 
 /* Run a particular program based on an activation */
@@ -386,6 +394,15 @@ update_appointment_menu_items_idle (gpointer user_data)
 	return FALSE;
 }
 
+static void
+hide_all_appointments (void)
+{
+	int i;
+
+	for (i=0; i<MAX_APPOINTMENT_MENUITEMS; i++)
+		dbusmenu_menuitem_property_set_bool(appointments[i], DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
+}
+
 static gboolean
 month_changed_cb (DbusmenuMenuitem * menuitem, gchar *name, GVariant *variant, guint timestamp)
 {
@@ -398,11 +415,7 @@ month_changed_cb (DbusmenuMenuitem * menuitem, gchar *name, GVariant *variant, g
 	   user. */
 	dbusmenu_menuitem_property_remove(menuitem, CALENDAR_MENUITEM_PROP_MARKS);
 
-	GList * appointment;
-	for (appointment = appointments; appointment != NULL; appointment = g_list_next(appointment)) {
-		DbusmenuMenuitem * mi = DBUSMENU_MENUITEM(appointment->data);
-		dbusmenu_menuitem_property_set_bool(mi, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-	}
+	hide_all_appointments ();
 
 	g_idle_add(update_appointment_menu_items_idle, NULL);
 	return TRUE;
@@ -433,11 +446,7 @@ day_selected_cb (DbusmenuMenuitem * menuitem, gchar *name, GVariant *variant, gu
 		}
 	}
 
-	GList * appointment;
-	for (appointment = appointments; appointment != NULL; appointment = g_list_next(appointment)) {
-		DbusmenuMenuitem * mi = DBUSMENU_MENUITEM(appointment->data);
-		dbusmenu_menuitem_property_set_bool(mi, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-	}
+	hide_all_appointments ();
 
 	start_time_appointments = new_time;
 
@@ -506,17 +515,7 @@ show_events_changed (void)
 	} else {
 		dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		dbusmenu_menuitem_property_set_bool(events_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-		/* Remove all of the previous appointments */
-		if (appointments != NULL) {
-			g_debug("Hiding old appointments");
-			GList * appointment;
-			for (appointment = appointments; appointment != NULL; appointment = g_list_next(appointment)) {
-				DbusmenuMenuitem * litem =  DBUSMENU_MENUITEM(appointment->data);
-				g_debug("Hiding old appointment: %p", litem);
-				// Remove all the existing menu items which are in appointments.
-				dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(litem), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-			}
-		}
+		hide_all_appointments ();
 		stop_ecal_timer();
 	}
 }
@@ -531,12 +530,8 @@ calendar_app_is_usable (void)
 	g_debug ("found calendar app: '%s'", evo);
 	g_free (evo);
 
-	/* confirm that it's got an account set up... */
-	GSList *accounts_list = gconf_client_get_list (gconf, "/apps/evolution/mail/accounts", GCONF_VALUE_STRING, NULL);
-	const guint n = g_slist_length (accounts_list);
-	g_debug ("found %u evolution accounts", n);
-	g_slist_free_full (accounts_list, g_free);
-	return n > 0;
+	/* see if there are any calendar sources */
+	return appointment_sources > 0;
 }
 
 /* Looks for the calendar application and enables the item if
@@ -549,19 +544,32 @@ check_for_calendar (gpointer user_data)
 	dbusmenu_menuitem_property_set_bool(date, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 	
 	if (!get_greeter_mode () && calendar_app_is_usable()) {
+
+		int i;
+		int pos = 2;
 		
 		g_signal_connect (G_OBJECT(date), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
 		                  G_CALLBACK (activate_cb), "evolution -c calendar");
 		
 		events_separator = dbusmenu_menuitem_new();
 		dbusmenu_menuitem_property_set(events_separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
-		dbusmenu_menuitem_child_add_position(root, events_separator, 2);
+		dbusmenu_menuitem_child_add_position(root, events_separator, pos++);
+
+		for (i=0; i<MAX_APPOINTMENT_MENUITEMS; i++)
+		{
+			DbusmenuMenuitem * item = dbusmenu_menuitem_new();
+			dbusmenu_menuitem_property_set (item, DBUSMENU_MENUITEM_PROP_TYPE, APPOINTMENT_MENUITEM_TYPE);
+			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
+			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+			appointments[i] = item;
+			dbusmenu_menuitem_child_add_position(root, item, pos++);
+		}
+
 		add_appointment = dbusmenu_menuitem_new();
 		dbusmenu_menuitem_property_set (add_appointment, DBUSMENU_MENUITEM_PROP_LABEL, _("Add Event…"));
 		dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 		g_signal_connect(G_OBJECT(add_appointment), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), "evolution -c calendar");
-		dbusmenu_menuitem_child_add_position (root, add_appointment, 3);
-
+		dbusmenu_menuitem_child_add_position (root, add_appointment, pos++);
 
 		if (g_settings_get_boolean(conf, SETTINGS_SHOW_EVENTS_S)) {
 			dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
@@ -579,8 +587,10 @@ check_for_calendar (gpointer user_data)
 		g_signal_connect(calendar, "event::day-selected-double-click", G_CALLBACK(day_selected_double_click_cb), NULL);
 	} else {
 		g_debug("Unable to find calendar app.");
-		dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-		dbusmenu_menuitem_property_set_bool(events_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+		if (add_appointment != NULL)
+			dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+		if (events_separator != NULL)
+			dbusmenu_menuitem_property_set_bool(events_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 	}
 	
 	if (g_settings_get_boolean(conf, SETTINGS_SHOW_CALENDAR_S)) {
@@ -594,27 +604,6 @@ check_for_calendar (gpointer user_data)
 	return FALSE;
 }
 
-// Authentication function
-static gchar *
-auth_func (ECal *ecal, 
-           const gchar *prompt, 
-           const gchar *key, 
-           gpointer user_data)
-{
-	ESource *source = e_cal_get_source (ecal);
-	gchar *auth_domain = e_source_get_duped_property (source, "auth-domain");
-
-	const gchar *component_name;
-	if (auth_domain) component_name = auth_domain;
-	else component_name = "Calendar";
-	
-	gchar *password = e_passwords_get_password (component_name, key);
-	
-	g_free (auth_domain);
-
-	return password;
-}
-
 static gint
 compare_comp_instances (gconstpointer ga, gconstpointer gb)
 {
@@ -654,18 +643,24 @@ populate_appointment_instances (ECalComponent * comp,
                                 time_t          end,
                                 gpointer        data)
 {
-	g_debug("Appending item %p", comp);
-	
-	ECalComponentVType vtype = e_cal_component_get_vtype (comp);
-	if (vtype != E_CAL_COMPONENT_EVENT && vtype != E_CAL_COMPONENT_TODO) return FALSE;
-	
-	icalproperty_status status;
-	e_cal_component_get_status (comp, &status);
-	if (status == ICAL_STATUS_COMPLETED || status == ICAL_STATUS_CANCELLED) return FALSE;
+	const ECalComponentVType vtype = e_cal_component_get_vtype (comp);
 
-	struct comp_instance *ci = comp_instance_new (comp, start, end, E_SOURCE(data));
-	comp_instances = g_list_append (comp_instances, ci);
-	return TRUE;
+	if ((vtype == E_CAL_COMPONENT_EVENT) || (vtype == E_CAL_COMPONENT_TODO))
+	{
+		icalproperty_status status;
+		e_cal_component_get_status (comp, &status);
+
+		if ((status != ICAL_STATUS_COMPLETED) && (status != ICAL_STATUS_CANCELLED))
+		{
+			gchar * str = e_cal_component_get_as_string (comp);
+			g_debug("Appending item %s", str);
+			struct comp_instance *ci = comp_instance_new (comp, start, end, E_SOURCE(data));
+			comp_instances = g_list_append (comp_instances, ci);
+			g_free (str);
+		}
+	}
+
+	return TRUE; /* tell eds to keep iterating */
 }
 
 /* Populate the menu with todays, next 5 appointments. 
@@ -674,7 +669,7 @@ populate_appointment_instances (ECalComponent * comp,
  * this is a problem mainly on the EDS side of things, not ours. 
  */
 static gboolean
-update_appointment_menu_items (gpointer user_data)
+update_appointment_menu_items (gpointer unused)
 {
 	// FFR: we should take into account short term timers, for instance
 	// tea timers, pomodoro timers etc... that people may add, this is hinted to in the spec.
@@ -685,12 +680,11 @@ update_appointment_menu_items (gpointer user_data)
 	updating_appointments = TRUE;
 	
 	time_t curtime = 0, t1 = 0, t2 = 0;
-	GList *l;
-	GSList *g;
+	GList *l, *s;
 	GError *gerror = NULL;
 	gint i;
 	gint width = 0, height = 0;
-	ESourceList * sources = NULL;
+	GList * sources = NULL;
 
 	// Get today & work out query times
 	time(&curtime);
@@ -730,84 +724,50 @@ update_appointment_menu_items (gpointer user_data)
 	highlightdays = highlightdays + 7; // Minimum of 7 days ahead 
 	t2 = t1 + (time_t) (highlightdays * 24 * 60 * 60);
 	
-	if (!e_cal_get_sources(&sources, E_CAL_SOURCE_TYPE_EVENT, &gerror)) {
-		g_debug("Failed to get ecal sources\n");
-		g_clear_error (&gerror);
-		return FALSE;
-	}
-	
 	// clear any previous comp_instances
 	g_list_free_full (comp_instances, (GDestroyNotify)comp_instance_free);
 	comp_instances = NULL;
 
-	GSList *cal_list = gconf_client_get_list(gconf, "/apps/evolution/calendar/display/selected_calendars", GCONF_VALUE_STRING, &gerror);
-	if (gerror) {
-	  g_debug("Failed to get evolution preference for enabled calendars");
-	  g_clear_error (&gerror);
-	  cal_list = NULL;
-	}
-	
 	// Generate instances for all sources
-	for (g = e_source_list_peek_groups (sources); g; g = g->next) {
-		ESourceGroup *group = E_SOURCE_GROUP (g->data);
-		GSList *s;
-		
-		for (s = e_source_group_peek_sources (group); s; s = s->next) {
-			ESource *source = E_SOURCE (s->data);
-			g_signal_connect (G_OBJECT(source), "changed", G_CALLBACK (update_appointment_menu_items), NULL);
-			ECal *ecal = e_cal_new(source, E_CAL_SOURCE_TYPE_EVENT);
-			e_cal_set_auth_func (ecal, (ECalAuthFunc) auth_func, NULL);
-			
-			icaltimezone* current_zone = icaltimezone_get_builtin_timezone(current_timezone);
-			if (!current_zone) {
-				// current_timezone may be a TZID?
-				current_zone = icaltimezone_get_builtin_timezone_from_tzid(current_timezone);
-			}
-			if (current_zone && !e_cal_set_default_timezone(ecal, current_zone, &gerror)) {
-				g_debug("Failed to set ecal default timezone %s", gerror->message);
-				g_clear_error (&gerror);
-				g_object_unref(ecal);
-				continue;
-			}
-			
-			if (!e_cal_open(ecal, FALSE, &gerror)) {
-				g_debug("Failed to get ecal sources %s", gerror->message);
+	for (s=appointment_sources; s!=NULL; s=s->next) {
+
+               ESource *source = E_SOURCE (s->data);
+               ECalClient *ecal = e_cal_client_new(source, E_CAL_CLIENT_SOURCE_TYPE_EVENTS, &gerror);
+
+               icaltimezone* current_zone = icaltimezone_get_builtin_timezone(current_timezone);
+               if (!current_zone) {
+                       // current_timezone may be a TZID?
+                       current_zone = icaltimezone_get_builtin_timezone_from_tzid(current_timezone);
+               }
+
+               e_cal_client_set_default_timezone (ecal, current_zone);
+
+               g_debug("Checking if source %s is enabled", e_source_get_uid(source));
+               if (e_source_get_enabled (source)) {
+                       g_debug("source is enabled, generating instances");
+
+                       if (!e_client_open_sync (E_CLIENT (ecal), TRUE, NULL, &gerror)) {
+                               g_debug("Failed to open source: %s", gerror->message);
 				g_clear_error (&gerror);
 				g_object_unref(ecal);
 				continue;
 			}
 
-			const gchar *ecal_uid = e_source_peek_uid(source);
-			g_debug("Checking ecal_uid is enabled: %s", ecal_uid);
-			const gboolean in_list = g_slist_find_custom (cal_list, ecal_uid, (GCompareFunc)g_strcmp0) != NULL;
-			if (!in_list) {
-				g_object_unref(ecal);
-				continue;
-			}
-
-			g_debug("ecal_uid is enabled, generating instances");
-			e_cal_generate_instances (ecal, t1, t2, (ECalRecurInstanceFn) populate_appointment_instances, source);
-			g_object_unref(ecal);
-		}
-	}
-	g_slist_free_full (cal_list, g_free);
+                       e_cal_client_generate_instances_sync (ecal,
+                                                             t1,
+                                                             t2,
+                                                             populate_appointment_instances,
+                                                             source);
+               }
+               g_object_unref(ecal);
+       }
 
 	g_debug("Number of ECalComponents returned: %d", g_list_length(comp_instances));
 	GList *sorted_comp_instances = g_list_sort(comp_instances, compare_comp_instances);
 	comp_instances = NULL;
 	g_debug("Components sorted");
-	
-	/* Hiding all of the previous appointments */
-	if (appointments != NULL) {
-		g_debug("Hiding old appointments");
-		GList * appointment;
-		for (appointment = appointments; appointment != NULL; appointment = g_list_next(appointment)) {
-			DbusmenuMenuitem * litem =  DBUSMENU_MENUITEM(appointment->data);
-			g_debug("Hiding old appointment: %p", litem);
-			// Remove all the existing menu items which are in appointments.
-			dbusmenu_menuitem_property_set_bool(DBUSMENU_MENUITEM(litem), DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-		}
-	}
+
+	hide_all_appointments ();	
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 	if (width <= 0) width = 12;
@@ -832,7 +792,6 @@ update_appointment_menu_items (gpointer user_data)
 	g_variant_builder_init (&markeddays, G_VARIANT_TYPE ("ai"));
 	
 	i = 0;
-	GList * cached_appointment = appointments;
 	for (l = sorted_comp_instances; l; l = l->next) {
 		struct comp_instance *ci = l->data;
 		ECalComponent *ecalcomp = ci->comp;
@@ -864,28 +823,18 @@ update_appointment_menu_items (gpointer user_data)
 		} else if (vtype == E_CAL_COMPONENT_TODO) {
 			if (ci->end < start_time_appointments) continue;
 		}
-		
-		if (i >= 5) continue;
+	
+		if (i >= MAX_APPOINTMENT_MENUITEMS)
+			continue;
+
 		i++;
+		item = appointments[i];
 
-		if (cached_appointment == NULL) {
-			g_debug("Create menu item");
-			
-			item = dbusmenu_menuitem_new();
-			dbusmenu_menuitem_property_set       (item, DBUSMENU_MENUITEM_PROP_TYPE, APPOINTMENT_MENUITEM_TYPE);
+		/* Remove the icon as we might not replace it on error */
+		dbusmenu_menuitem_property_remove(item, APPOINTMENT_MENUITEM_PROP_ICON);
 
-			dbusmenu_menuitem_child_add_position (root, item, 2+i);
-			appointments = g_list_append (appointments, item); // Keep track of the items here to make them easy to remove
-		} else {
-			item = DBUSMENU_MENUITEM(cached_appointment->data);
-			cached_appointment = g_list_next(cached_appointment);
-
-			/* Remove the icon as we might not replace it on error */
-			dbusmenu_menuitem_property_remove(item, APPOINTMENT_MENUITEM_PROP_ICON);
-
-			/* Remove the activate handler */
-			g_signal_handlers_disconnect_matched(G_OBJECT(item), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK(activate_cb), NULL);
-		}
+		/* Remove the activate handler */
+		g_signal_handlers_disconnect_matched(G_OBJECT(item), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK(activate_cb), NULL);
 
 		dbusmenu_menuitem_property_set_bool  (item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 		dbusmenu_menuitem_property_set_bool  (item, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
@@ -942,7 +891,7 @@ update_appointment_menu_items (gpointer user_data)
 		                       G_CALLBACK(activate_cb), cmd, (GClosureNotify)g_free, 0);
 		g_free (ad);
 
-        const gchar *color_spec = e_source_peek_color_spec(ci->source);
+	const gchar *color_spec = e_source_selectable_get_color (e_source_get_extension (ci->source, E_SOURCE_EXTENSION_CALENDAR));
         g_debug("Colour to use: %s", color_spec);
 			
 		// Draw the correct icon for the appointment type and then tint it using mask fill.
@@ -950,17 +899,10 @@ update_appointment_menu_items (gpointer user_data)
         if (color_spec != NULL) {
         	g_debug("Creating a cairo surface: size, %d by %d", width, height);         
         	cairo_surface_t *surface = cairo_image_surface_create( CAIRO_FORMAT_ARGB32, width, height ); 
-
 			cairo_t *cr = cairo_create(surface);
-#if GTK_CHECK_VERSION(3,0,0)
         	GdkRGBA rgba;
         	if (gdk_rgba_parse (&rgba, color_spec))
         		gdk_cairo_set_source_rgba (cr, &rgba);
-#else
-			GdkColor color;
-			if (gdk_color_parse (color_spec, &color))
-				gdk_cairo_set_source_color (cr, &color);
-#endif
 			cairo_paint(cr);
     		cairo_set_source_rgba(cr, 0,0,0,0.5);
     		cairo_set_line_width(cr, 1);
@@ -1095,6 +1037,7 @@ build_menus (DbusmenuMenuitem * root)
 		update_location_menu_items();
 	
 		g_signal_connect (conf, "changed::" SETTINGS_SHOW_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
+		g_signal_connect (conf, "changed::" SETTINGS_SHOW_DETECTED_S, G_CALLBACK (show_locations_changed), NULL);
 		g_signal_connect (conf, "changed::" SETTINGS_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
 		g_signal_connect (conf, "changed::" SETTINGS_SHOW_EVENTS_S, G_CALLBACK (show_events_changed), NULL);
 		g_signal_connect (conf, "changed::" SETTINGS_TIME_FORMAT_S, G_CALLBACK (time_format_changed), NULL);
@@ -1115,14 +1058,26 @@ build_menus (DbusmenuMenuitem * root)
 	return;
 }
 
+static void
+on_clock_skew (void)
+{
+	/* tell the indicators to refresh */
+	if (IS_DATETIME_INTERFACE (dbus))
+		datetime_interface_update (DATETIME_INTERFACE(dbus));
+
+	/* update our day label */
+	update_datetime (NULL);
+	day_timer_reset();
+
+	return;
+}
+
 /* Run when the timezone file changes */
 static void
 timezone_changed (GFileMonitor * monitor, GFile * file, GFile * otherfile, GFileMonitorEvent event, gpointer user_data)
 {
 	update_current_timezone();
-	datetime_interface_update(DATETIME_INTERFACE(user_data));
-	update_datetime(NULL);
-	setup_timer();
+	on_clock_skew();
 	return;
 }
 
@@ -1142,42 +1097,58 @@ build_timezone (DatetimeInterface * dbus)
 }
 
 /* Source ID for the timer */
-static guint timer = 0;
+static guint day_timer = 0;
 
 /* Execute at a given time, update and setup a new
    timer to go again.  */
 static gboolean
-timer_func (gpointer user_data)
+day_timer_func (gpointer user_data)
 {
-	timer = 0;
+	day_timer = 0;
 	/* Reset up each time to reduce error */
-	setup_timer();
+	day_timer_reset();
 	update_datetime(NULL);
-	return FALSE;
+	return G_SOURCE_REMOVE;
 }
 
 /* Sets up the time to launch the timer to update the
    date in the datetime entry */
 static void
-setup_timer (void)
+day_timer_reset (void)
 {
-	if (timer != 0) {
-		g_source_remove(timer);
-		timer = 0;
+	if (day_timer != 0) {
+		g_source_remove(day_timer);
+		day_timer = 0;
 	}
 
 	time_t t;
 	t = time(NULL);
 	struct tm * ltime = localtime(&t);
 
-	timer = g_timeout_add_seconds(((23 - ltime->tm_hour) * 60 * 60) +
-	                              ((59 - ltime->tm_min) * 60) +
-	                              ((60 - ltime->tm_sec)) + 60 /* one minute past */,
-	                              timer_func, NULL);
+	day_timer = g_timeout_add_seconds(((23 - ltime->tm_hour) * 60 * 60) +
+	                                  ((59 - ltime->tm_min) * 60) +
+	                                  ((60 - ltime->tm_sec)) + 60 /* one minute past */,
+	                                  day_timer_func, NULL);
 
 	return;
 }
 
+static gboolean
+skew_check_timer_func (gpointer unused G_GNUC_UNUSED)
+{
+	static time_t prev_time = 0;
+	const time_t cur_time = time (NULL);
+	const double diff_sec = fabs (difftime (cur_time, prev_time));
+
+	if (prev_time && (diff_sec > SKEW_DIFF_THRESHOLD_SEC)) {
+		g_debug (G_STRLOC" clock skew detected (%.0f seconds)", diff_sec);
+		on_clock_skew ();
+	}
+
+	prev_time = cur_time;
+	return G_SOURCE_CONTINUE;
+}
+
 static void
 session_active_change_cb (GDBusProxy * proxy, gchar * sender_name, gchar * signal_name,
                           GVariant * parameters, gpointer user_data)
@@ -1187,9 +1158,7 @@ session_active_change_cb (GDBusProxy * proxy, gchar * sender_name, gchar * signa
 		gboolean idle = FALSE;
 		g_variant_get(parameters, "(b)", &idle);
 		if (!idle) {
-			datetime_interface_update(DATETIME_INTERFACE(user_data));
-			update_datetime(NULL);
-			setup_timer();
+			on_clock_skew ();
 		}
 	}
 	return;
@@ -1266,7 +1235,6 @@ geo_client_clean (void)
 	}
 
 	g_signal_handlers_disconnect_by_func(G_OBJECT(geo_master), geo_client_invalid, NULL);
-	g_signal_handlers_disconnect_by_func(G_OBJECT(geo_master), geo_address_change, NULL);
 	g_object_unref(G_OBJECT(geo_master));
 
 	geo_master = NULL;
@@ -1338,28 +1306,6 @@ geo_client_invalid (GeoclueMasterClient * client, gpointer user_data)
 	return;
 }
 
-/* Address provider changed, we need to get that one */
-static void
-geo_address_change (GeoclueMasterClient * client, gchar * a, gchar * b, gchar * c, gchar * d, gpointer user_data)
-{
-	g_warning("Address provider changed.  Let's change");
-
-	/* If the address is supposed to have changed we need to drop the old
-	   address before starting to get the new one. */
-	geo_address_clean();
-
-	geoclue_master_client_create_address_async(geo_master, geo_create_address, NULL);
-
-	if (geo_timezone != NULL) {
-		g_free(geo_timezone);
-		geo_timezone = NULL;
-	}
-
-	update_location_menu_items();
-
-	return;
-}
-
 /* Callback from creating the client */
 static void
 geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar * path, GError * error, gpointer user_data)
@@ -1395,7 +1341,6 @@ geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar *
 	geoclue_master_client_create_address_async(geo_master, geo_create_address, NULL);
 
 	g_signal_connect(G_OBJECT(client), "invalidated", G_CALLBACK(geo_client_invalid), NULL);
-	g_signal_connect(G_OBJECT(client), "address-provider-changed", G_CALLBACK(geo_address_change), NULL);
 
 	return;
 }
@@ -1418,6 +1363,34 @@ service_shutdown (IndicatorService * service, gpointer user_data)
 	return;
 }
 
+static void
+free_appointment_sources (void)
+{
+	g_list_free_full (appointment_sources, g_object_unref);
+	appointment_sources = NULL;
+}
+
+static void
+init_appointment_sources (void)
+{
+	GList * l;
+
+	appointment_sources = e_source_registry_list_sources (source_registry, E_SOURCE_EXTENSION_CALENDAR);
+
+	for (l=appointment_sources; l!=NULL; l=l->next)
+		g_signal_connect (G_OBJECT(l->data), "changed", G_CALLBACK (update_appointment_menu_items), NULL);
+}
+
+/* rebuilds both the appointment sources and menu */
+static void
+update_appointments (void)
+{
+	free_appointment_sources ();
+	init_appointment_sources ();
+
+	update_appointment_menu_items (NULL);
+}
+
 /* Function to build everything up.  Entry point from asm. */
 int
 main (int argc, char ** argv)
@@ -1436,10 +1409,21 @@ main (int argc, char ** argv)
 
 	/* Set up GSettings */
 	conf = g_settings_new(SETTINGS_INTERFACE);
-	/* Set up gconf for getting evolution enabled calendars */
-	gconf = gconf_client_get_default();
 	// TODO Add a signal handler to catch gsettings changes and respond to them
 
+	/* Build our list of appointment calendar sources.
+	   When a source changes, update our menu items.
+	   When sources are added or removed, update our list and menu items. */
+	source_registry = e_source_registry_new_sync (NULL, NULL);
+	g_object_connect (source_registry,
+	                  "signal::source-added", update_appointments,
+	                  "signal::source-removed", update_appointments,
+	                  "signal::source-changed", update_appointment_menu_items,
+	                  "signal::source-disabled", update_appointment_menu_items,
+	                  "signal::source-enabled", update_appointment_menu_items,
+	                  NULL);
+	init_appointment_sources ();
+
 	/* Building the base menu */
 	server = dbusmenu_server_new(MENU_OBJ);
 	root = dbusmenu_menuitem_new();
@@ -1460,8 +1444,13 @@ main (int argc, char ** argv)
 	/* Setup timezone watch */
 	build_timezone(dbus);
 
-	/* Setup the timer */
-	setup_timer();
+	/* Set up the day timer */
+	day_timer_reset();
+
+	/* Set up the skew-check timer */
+	g_timeout_add_seconds (SKEW_CHECK_INTERVAL_SEC,
+	                       skew_check_timer_func,
+	                       NULL);
 
 	/* And watch for system resumes */
 	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
@@ -1475,12 +1464,15 @@ main (int argc, char ** argv)
 	mainloop = g_main_loop_new(NULL, FALSE);
 	g_main_loop_run(mainloop);
 
+	free_appointment_sources();
+
 	g_object_unref(G_OBJECT(conf));
 	g_object_unref(G_OBJECT(master));
 	g_object_unref(G_OBJECT(dbus));
 	g_object_unref(G_OBJECT(service));
 	g_object_unref(G_OBJECT(server));
 	g_object_unref(G_OBJECT(root));
+	g_object_unref(G_OBJECT(source_registry));
 
 	icaltimezone_free_builtin_timezones();
 
