@@ -154,6 +154,7 @@ GType indicator_datetime_get_type (void) G_GNUC_CONST;
 
 static void indicator_datetime_class_init (IndicatorDatetimeClass *klass);
 static void indicator_datetime_init       (IndicatorDatetime *self);
+static void timezone_update_all_labels    (IndicatorDatetime *self);
 static void set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec);
 static void get_property (GObject * object, guint prop_id, GValue * value, GParamSpec * pspec);
 static void indicator_datetime_dispose    (GObject *object);
@@ -267,18 +268,30 @@ indicator_datetime_class_init (IndicatorDatetimeClass *klass)
 static void
 menu_visible_notfy_cb(GtkWidget * menu, G_GNUC_UNUSED GParamSpec *pspec, gpointer user_data)
 {
+	GtkWidget * w;
+	GtkCalendar * calendar;
 	IndicatorDatetime * self = INDICATOR_DATETIME(user_data);
 	GDateTime *datetime;
-	gint y, m, d;
+	gint cur_y, cur_m, cur_d;
+	guint cal_y, cal_m, cal_d;
 
 	g_debug("notify visible signal received");
 
+	/* set the calendar to today's date */
 	datetime = g_date_time_new_now_local ();
-	g_date_time_get_ymd (datetime, &y, &m, &d);
+	g_date_time_get_ymd (datetime, &cur_y, &cur_m, &cur_d);
 	g_date_time_unref (datetime);
+	w = ido_calendar_menu_item_get_calendar (self->priv->ido_calendar);
+	calendar = GTK_CALENDAR(w);
+	gtk_calendar_get_date (calendar, &cal_y, &cal_m, &cal_d);
+	if ((cur_y != cal_y) || (cur_m-1 != cal_m))
+		gtk_calendar_select_month (calendar, cur_m-1, cur_y); /* (cur_m is 1-based) */
+	if (cur_d != cal_d)
+		gtk_calendar_select_day (calendar, cur_d);
 
-  	// Set the calendar to todays date
-	ido_calendar_menu_item_set_date (self->priv->ido_calendar, y, m-1, d);
+	/* Update in case date was changed outside of indicator-datetime */
+	update_label(self, NULL);
+	timezone_update_all_labels(self);
 
 	// Make sure the day-selected signal is sent so the menu updates - may duplicate
 	/*GVariant *variant = g_variant_new_uint32((guint)curtime);
@@ -1220,11 +1233,7 @@ new_appointment_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, Dbu
 
 	mi_data->gmi = gtk_menu_item_new();
 
-#if GTK_CHECK_VERSION(3,0,0)
 	GtkWidget * hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-#else
-	GtkWidget * hbox = gtk_hbox_new(FALSE, 4);
-#endif
 
 	/* Icon, probably someone's face or avatar on an IM */
 	mi_data->icon = gtk_image_new();
@@ -1435,11 +1444,7 @@ new_timezone_item(DbusmenuMenuitem * newitem,
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi_data->gmi),
 		dbusmenu_menuitem_property_get_bool(newitem, TIMEZONE_MENUITEM_PROP_RADIO));
 
-#if GTK_CHECK_VERSION(3,0,0)
 	GtkWidget * hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-#else
-	GtkWidget * hbox = gtk_hbox_new(FALSE, 4);
-#endif
 
   	/* Label, probably a username, chat room or mailbox name */
 	mi_data->label = gtk_label_new("");
